@@ -1,4 +1,4 @@
-/*      $Id: config_file.c,v 5.20 2005/06/18 20:39:45 lirc Exp $      */
+/*      $Id: config_file.c,v 5.26 2007/03/10 21:59:44 lirc Exp $      */
 
 /****************************************************************************
  ** config_file.c ***********************************************************
@@ -401,9 +401,14 @@ int defineRemote(char * key, char * val, char *val2, struct ir_remote *rem)
 		rem->repeat_gap=s_strtoul(val);
 		return(1);
 	}
+       	/* obsolete: use toggle_bit_mask instead */
 	else if (strcasecmp("toggle_bit",key)==0){
-		rem->toggle_bit=s_strtoi(val);
-		return(1);
+		rem->toggle_bit = s_strtoi(val);
+		return 1;
+	}
+	else if (strcasecmp("toggle_bit_mask",key)==0){
+		rem->toggle_bit_mask = s_strtocode(val);
+		return 1;
 	}
 	else if (strcasecmp("toggle_mask",key)==0){
 		rem->toggle_mask=s_strtocode(val);
@@ -422,6 +427,10 @@ int defineRemote(char * key, char * val, char *val2, struct ir_remote *rem)
 		rem->min_repeat=s_strtoi(val);
 		return(1);
 	}
+	else if (strcasecmp("min_code_repeat",key)==0){
+		rem->min_code_repeat=s_strtoi(val);
+		return(1);
+	}
 	else if (strcasecmp("frequency",key)==0){
 		rem->freq=s_strtoui(val);
 		return(1);
@@ -533,6 +542,103 @@ int defineRemote(char * key, char * val, char *val2, struct ir_remote *rem)
 	return(0);
 }
     
+static int sanityChecks(struct ir_remote *rem)
+{
+	struct ir_ncode *codes;
+	struct ir_code_node *node;
+	
+	if (!rem->name)
+	{
+		logprintf(LOG_ERR,"you must specify a remote name");
+		return 0;
+	}
+	if(rem->gap == 0)
+	{
+		logprintf(LOG_WARNING, "you should specify a valid gap value");
+	}
+	if(has_repeat_gap(rem) && is_const(rem))
+	{
+		logprintf(LOG_WARNING, "repeat_gap will be ignored if "
+			  "CONST_LENGTH flag is set");
+	}
+
+	if(is_raw(rem)) return 1;
+
+	if((rem->pre_data&gen_mask(rem->pre_data_bits)) != rem->pre_data)
+	{
+		logprintf(LOG_WARNING, "invalid pre_data found for %s",
+			  rem->name);
+		rem->pre_data &= gen_mask(rem->pre_data_bits);
+	}
+	if((rem->post_data&gen_mask(rem->post_data_bits)) != rem->post_data)
+	{
+		logprintf(LOG_WARNING, "invalid post_data found for %s",
+			  rem->name);
+		rem->post_data &= gen_mask(rem->post_data_bits);
+	}
+	for(codes = rem->codes; codes->name != NULL; codes++)
+	{
+		if((codes->code&gen_mask(rem->bits)) != codes->code)
+		{
+			logprintf(LOG_WARNING, "invalid code found for %s: %s",
+				  rem->name, codes->name);
+			codes->code &= gen_mask(rem->bits);
+		}
+		for(node = codes->next; node != NULL; node = node->next)
+		{
+			if((node->code&gen_mask(rem->bits)) != node->code)
+			{
+				logprintf(LOG_WARNING, "invalid code found "
+					 "for %s: %s",
+					  rem->name, codes->name);
+				node->code &= gen_mask(rem->bits);
+			}
+		}
+	}
+	
+	return 1;
+}
+
+struct ir_remote *sort_by_bit_count(struct ir_remote *remotes)
+{
+	struct ir_remote *top, *rem, *next, *prev, *scan;
+	
+	rem = remotes;
+	top = NULL;
+	while(rem!=NULL)
+	{
+		next = rem->next;
+		
+		scan = top;
+		prev = NULL;
+		while(scan && bit_count(scan)<=bit_count(rem))
+		{
+			prev = scan;
+			scan = scan->next;
+		}
+		if(prev)
+		{
+			prev->next = rem;
+		}
+		else
+		{
+			top = rem;
+		}
+		if(scan)
+		{
+			rem->next = scan;
+		}
+		else
+		{
+			rem->next = NULL;
+		}
+		
+		rem = next;
+	}
+	
+	return top;
+}
+
 struct ir_remote * read_config(FILE *f)
 {
 	char buf[LINE_LEN+1], *key, *val, *val2;
@@ -686,11 +792,10 @@ struct ir_remote * read_config(FILE *f)
 					/* print_remote(rem); */
                                         if (!checkMode(mode,ID_remote,
                                                   "end remote")) break;
-                                        if (!rem->name){
-                                                logprintf(LOG_ERR,"you must specify a remote name");
+					if(!sanityChecks(rem)) {
                                                 parse_error=1;
                                                 break;
-                                        }
+					}
 
 #                                       ifdef DYNCODES
 					if(rem->dyncodes_name==NULL)
@@ -705,11 +810,6 @@ struct ir_remote * read_config(FILE *f)
                                         rem->next=NULL;
 					rem->last_code=NULL;
                                         mode=ID_none;     /* switch back */
-					if(has_repeat_gap(rem) && 
-					   is_const(rem))
-					{
-						logprintf(LOG_WARNING,"repeat_gap will be ignored if CONST_LENGTH flag is set");
-					}
 				}else if(mode==ID_codes){
 					code=defineCode(key, val, &name_code);
 					while(!parse_error && val2!=NULL)
@@ -883,12 +983,38 @@ struct ir_remote * read_config(FILE *f)
 		}
 		if(rem->flags&RC6 && rem->rc6_mask==0 && rem->toggle_bit>0)
 		{
-			int all_bits=rem->pre_data_bits+
-				rem->bits+
-				rem->post_data_bits;
+			int all_bits=bit_count(rem);
 			
 			rem->rc6_mask=((ir_code) 1)<<(all_bits-rem->toggle_bit);
 		}
+		if(rem->toggle_bit > 0)
+		{
+			int all_bits=bit_count(rem);
+			
+			if(has_toggle_bit_mask(rem))
+			{
+				logprintf(LOG_WARNING,
+					  "%s uses both toggle_bit and "
+					  "toggle_bit_mask", rem->name);
+			}
+			else
+			{
+				rem->toggle_bit_mask=((ir_code) 1)<<(all_bits-rem->toggle_bit);
+			}
+			rem->toggle_bit = 0;
+		}
+		if(has_toggle_bit_mask(rem))
+		{
+			if(!is_raw(rem) && rem->codes)
+			{
+				rem->toggle_bit_mask_state = (rem->codes->code & rem->toggle_bit_mask);
+				if(rem->toggle_bit_mask_state)
+				{
+					/* start with state set to 0 for backwards compatibility */
+					rem->toggle_bit_mask_state ^= rem->toggle_bit_mask;
+				}
+			}
+		}
 		if(is_serial(rem))
 		{
 			lirc_t base;
@@ -910,9 +1036,20 @@ struct ir_remote * read_config(FILE *f)
 				rem->bits_in_byte=8;
 			}
 		}
+		if(rem->min_code_repeat>0)
+		{
+			if(!has_repeat(rem) ||
+			   rem->min_code_repeat>rem->min_repeat)
+			{
+				logprintf(LOG_WARNING,
+					  "invalid min_code_repeat value");
+				rem->min_code_repeat = 0;
+			}
+		}
 		rem=rem->next;
 	}
 
+	top_rem = sort_by_bit_count(top_rem);
 #       if defined(DEBUG) && !defined(DAEMONIZE)
         /*fprint_remotes(stderr, top_rem);*/
 #       endif
