@@ -6,7 +6,9 @@
 #include "hardware.h"
 #include "hw-types.h"
 /* Hardware types */
+extern struct hardware hw_accent;
 extern struct hardware hw_alsa_usb;
+extern struct hardware hw_asusdh;
 extern struct hardware hw_atilibusb;
 extern struct hardware hw_audio;
 extern struct hardware hw_audio_alsa;
@@ -19,22 +21,27 @@ extern struct hardware hw_devinput;
 extern struct hardware hw_dsp;
 extern struct hardware hw_dvico;
 extern struct hardware hw_ea65;
+extern struct hardware hw_iguanaIR;
 extern struct hardware hw_irman;
 extern struct hardware hw_livedrive_midi;
 extern struct hardware hw_livedrive_seq;
 extern struct hardware hw_logitech;
+extern struct hardware hw_macmini;
 extern struct hardware hw_mouseremote;
 extern struct hardware hw_mouseremote_ps2;
 extern struct hardware hw_mp3anywhere;
 extern struct hardware hw_pcmak;
 extern struct hardware hw_pinsys;
 extern struct hardware hw_pixelview;
+extern struct hardware hw_sb0540;
 extern struct hardware hw_silitek;
 extern struct hardware hw_slinke;
 extern struct hardware hw_tira;
 extern struct hardware hw_udp;
 extern struct hardware hw_uirt2;
 extern struct hardware hw_uirt2_raw;
+extern struct hardware hw_usb_uirt_raw;
+extern struct hardware hw_usbx;
 
 
 #ifndef HW_DEFAULT
@@ -64,9 +71,11 @@ struct hardware hw_null=
 struct hardware *hw_list[] =
 {
 #ifdef LIRC_DRIVER_ANY
+	&hw_accent,
 #ifdef HAVE_ALSA_SB_RC
 	&hw_alsa_usb,
 #endif
+	&hw_asusdh,
 #ifdef HAVE_LIBUSB
 	&hw_atilibusb,
 #endif
@@ -91,6 +100,9 @@ struct hardware *hw_list[] =
 #endif
 	&hw_dvico,
 	&hw_ea65,
+#ifdef HAVE_IGUANAIR
+	&hw_iguanaIR,
+#endif
 #ifdef HAVE_LIBIRMAN
 	&hw_irman,
 #endif
@@ -104,12 +116,15 @@ struct hardware *hw_list[] =
 	&hw_pcmak,
 	&hw_pinsys,
 	&hw_pixelview,
+	&hw_sb0540,
 	&hw_silitek,
 	/*	&hw_slinke,*/
 	&hw_tira,
 	&hw_udp,
 	&hw_uirt2,
 	&hw_uirt2_raw,
+	&hw_usb_uirt_raw,
+	&hw_usbx,
 #else
 	&HW_DEFAULT,
 #endif
