@@ -1,4 +1,4 @@
-/*      $Id: hw_creative.c,v 5.5 2001/12/08 15:07:03 lirc Exp $      */
+/*      $Id: hw_creative.c,v 5.9 2007/03/10 20:15:49 lirc Exp $      */
 
 /****************************************************************************
  ** hw_creative.c ***********************************************************
@@ -40,33 +40,6 @@ struct timeval start,end,last;
 lirc_t gap,signal_length;
 ir_code pre,code;
 
-unsigned char mapping[]=
-{0xd8,
- 0x04,
- 0x38,
- 0xb4,
- 0x78,
- 0x98,
- 0xf8,
- 0x14,
- 0x84,
- 0x44,
- 0xc4,
- 0xf4,
- 0xa4,
- 0x64,
- 0xe4,
- 0x34,
- 0x94,
- 0x54,
- 0xd4,
- 0x0c,
- 0x24,
- 0x74,
- 0x58,
- 0xb8,
- 0x00};
-
 struct hardware hw_creative=
 {
 	LIRC_DRIVER_DEVICE,       /* default device */
@@ -81,6 +54,7 @@ struct hardware hw_creative=
 	NULL,                     /* send_func */
 	creative_rec,             /* rec_func */
 	creative_decode,          /* decode_func */
+	NULL,                     /* ioctl_func */
 	NULL,                     /* readdata */
 	"creative"
 };
@@ -193,9 +167,7 @@ char *creative_rec(struct ir_remote *remotes)
 			return(NULL);
 		}
 		if(b[0]!=0x4d ||
-		   b[1]!=0x05 ||
-		   b[4]!=0xac ||
-		   b[5]!=0x21)
+		   b[1]!=0x05 /* || b[4]!=0xac || b[5]!=0x21 */)
 		{
 			logprintf(LOG_ERR,"bad envelope");
 			return(NULL);
@@ -212,20 +184,9 @@ char *creative_rec(struct ir_remote *remotes)
 	}
 	gettimeofday(&end,NULL);
 
-	pre=0x8435;
-	for(i=0;mapping[i]!=0x00;i++)
-	{
-		if(mapping[i]==b[3])
-		{
-			code=(ir_code) (i<<8)|((~i)&0xff);
-			break;
-		}
-	}
-	if(mapping[i]==0x00)
-	{
-		logprintf(LOG_ERR,"unknown code");
-		return(NULL);
-	}
+	/* pre=0x8435; */
+	pre=reverse((((ir_code) b[4])<<8) | ((ir_code) b[5]),16);
+	code = reverse((((ir_code) b[2]) << 8) | ((ir_code) b[3]), 16);
 	
 	m=decode_all(remotes);
 	return(m);
