@@ -1,4 +1,4 @@
-/*      $Id: ir_remote.c,v 5.24 2005/10/25 19:05:35 lirc Exp $      */
+/*      $Id: ir_remote.c,v 5.31 2007/05/06 09:46:59 lirc Exp $      */
 
 /****************************************************************************
  ** ir_remote.c *************************************************************
@@ -27,6 +27,7 @@
 #include "lircd.h"
 #include "ir_remote.h"
 #include "hardware.h"
+#include "release.h"
 
 struct ir_remote *decoding=NULL;
 
@@ -71,6 +72,20 @@ void get_frequency_range(struct ir_remote *remotes,
 	}
 }
 
+struct ir_remote *is_in_remotes(struct ir_remote *remotes,
+				struct ir_remote *remote)
+{
+	while(remotes != NULL)
+	{
+		if(remotes == remote)
+		{
+			return remote;
+		}
+		remotes = remotes->next;
+	}
+	return NULL;
+}
+
 struct ir_remote *get_ir_remote(struct ir_remote *remotes,char *name)
 {
 	struct ir_remote *all;
@@ -106,53 +121,20 @@ struct ir_ncode *get_ir_code(struct ir_remote *remote,char *name)
 
 struct ir_ncode *get_code(struct ir_remote *remote,
 			  ir_code pre,ir_code code,ir_code post,
-			  int *repeat_statep)
+			  ir_code *toggle_bit_mask_statep)
 {
-	ir_code pre_mask,code_mask,post_mask;
-	int repeat_state, found_code;
+	ir_code pre_mask,code_mask,post_mask,toggle_bit_mask_state,all;
+	int found_code, have_code;
 	struct ir_ncode *codes,*found;
 	
 	pre_mask=code_mask=post_mask=0;
-	repeat_state=0;
-	if(remote->toggle_bit>0)
+
+	if(has_toggle_bit_mask(remote))
 	{
-		if(remote->toggle_bit<=remote->pre_data_bits)
-		{
-			repeat_state=
-			pre&(1<<(remote->pre_data_bits
-				 -remote->toggle_bit)) ? 1:0;
-			pre_mask=1<<(remote->pre_data_bits
-				     -remote->toggle_bit);
-		}
-		else if(remote->toggle_bit<=remote->pre_data_bits
-			+remote->bits)
-		{
-			repeat_state=
-			code&(1<<(remote->pre_data_bits
-				  +remote->bits
-				  -remote->toggle_bit)) ? 1:0;
-			code_mask=1<<(remote->pre_data_bits
-				      +remote->bits
-				      -remote->toggle_bit);
-		}
-		else if(remote->toggle_bit<=remote->pre_data_bits
-			+remote->bits
-			+remote->post_data_bits)
-		{
-			repeat_state=
-			post&(1<<(remote->pre_data_bits
-				  +remote->bits
-				  +remote->post_data_bits
-				  -remote->toggle_bit)) ? 1:0;
-			post_mask=1<<(remote->pre_data_bits
-				      +remote->bits
-				      +remote->post_data_bits
-				      -remote->toggle_bit);
-		}
-		else
-		{
-			logprintf(LOG_ERR,"bad toggle_bit");
-		}
+		pre_mask = remote->toggle_bit_mask >>
+			   (remote->bits + remote->post_data_bits);
+		post_mask = remote->toggle_bit_mask &
+		            gen_mask(remote->post_data_bits);
 	}
 	if(has_toggle_mask(remote) && remote->toggle_mask_state%2)
 	{
@@ -161,9 +143,7 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 		
 		affected=&post;
 		mask=remote->toggle_mask;
-		for(bit=current_bit=0;bit<remote->pre_data_bits+
-			    remote->bits+
-			    remote->post_data_bits;bit++,current_bit++)
+		for(bit=current_bit=0;bit<bit_count(remote);bit++,current_bit++)
 		{
 			if(bit==remote->post_data_bits)
 			{
@@ -209,14 +189,24 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 		}
 		LOGPRINTF(1,"post");
 	}
+
+	all = pre;
+	all <<= remote->bits;
+	all |= code;
+	all <<= remote->post_data_bits;
+	all |= post;
+	
+	toggle_bit_mask_state = all&remote->toggle_bit_mask;
+
 	found=NULL;
 	found_code=0;
+	have_code=0;
 	codes=remote->codes;
 	if(codes!=NULL)
 	{
 		while(codes->name!=NULL)
 		{
-			ir_code next_code;
+			ir_code next_code, next_all;
 			
 			if(codes->next!=NULL && codes->current!=NULL)
 			{
@@ -226,7 +216,13 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 			{
 				next_code=codes->code;
 			}
-			if((next_code|code_mask)==(code|code_mask))
+			next_all = remote->pre_data;
+			next_all <<= remote->bits;
+			next_all |= next_code;
+			next_all <<= remote->post_data_bits;
+			next_all |= remote->post_data;
+			if(next_all==all ||
+			   next_all==(all^remote->toggle_bit_mask))
 			{
 				found_code=1;
 				if(codes->next!=NULL)
@@ -241,10 +237,13 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 							codes->next->next;
 					}
 				}
-				found=codes;
-				if(codes->current==NULL)
+				if(!have_code)
 				{
-					break;
+					found=codes;
+					if(codes->current==NULL)
+					{
+						have_code=1;
+					}
 				}
 			}
 			else
@@ -257,13 +256,12 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 #       ifdef DYNCODES
 	if(!found_code)
 	{
-		if((remote->dyncodes[remote->dyncode].code|code_mask)!=
-		   (code|code_mask))
+		if(remote->dyncodes[remote->dyncode].code!=code)
 		{
 			remote->dyncode++;
 			remote->dyncode%=2;
 		}
-		remote->dyncodes[remote->dyncode].code=code&(~code_mask);
+		remote->dyncodes[remote->dyncode].code=code;
 		found=&(remote->dyncodes[remote->dyncode]);
 		found_code=1;
 	}
@@ -285,12 +283,12 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 			remote->toggle_code=NULL;
 		}
 	}
-	*repeat_statep=repeat_state;
+	*toggle_bit_mask_statep=toggle_bit_mask_state;
 	return(found);
 }
 
 unsigned long long set_code(struct ir_remote *remote,struct ir_ncode *found,
-			    int repeat_state,int repeat_flag,
+			    ir_code toggle_bit_mask_state,int repeat_flag,
 			    lirc_t remaining_gap)
 {
 	unsigned long long code;
@@ -300,11 +298,10 @@ unsigned long long set_code(struct ir_remote *remote,struct ir_ncode *found,
 
 	gettimeofday(&current,NULL);
 	if(remote==last_remote &&
-	   ((found->next==NULL && found==remote->last_code) || 
-	    (found->next!=NULL)) &&
+	   (found==remote->last_code || (found->next!=NULL && found->current!=NULL)) &&
 	   repeat_flag &&
 	   time_elapsed(&remote->last_send,&current)<1000000 &&
-	   (!(remote->toggle_bit>0) || repeat_state==remote->repeat_state))
+	   (!has_toggle_bit_mask(remote) || toggle_bit_mask_state==remote->toggle_bit_mask_state))
 	{
 		if(has_toggle_mask(remote))
 		{
@@ -335,13 +332,13 @@ unsigned long long set_code(struct ir_remote *remote,struct ir_ncode *found,
 			remote->toggle_mask_state=1;
 			remote->toggle_code=found;
 		}
-		if(remote->toggle_bit>0)
+		if(has_toggle_bit_mask(remote))
 		{
-			remote->repeat_state=repeat_state;
+			remote->toggle_bit_mask_state=toggle_bit_mask_state;
 		}
 	}
 	last_remote=remote;
-	remote->last_code=found;
+	if(found->current==NULL) remote->last_code=found;
 	remote->last_send=current;
 	remote->remaining_gap=remaining_gap;
 	
@@ -351,7 +348,7 @@ unsigned long long set_code(struct ir_remote *remote,struct ir_ncode *found,
 		code|=remote->pre_data;
 		code=code<<remote->bits;
 	}
-	code|=remote->last_code->code;
+	code|=found->code;
 	if(has_post(remote))
 	{
 		code=code<<remote->post_data_bits;
@@ -363,20 +360,47 @@ unsigned long long set_code(struct ir_remote *remote,struct ir_ncode *found,
 		   be rotated separately but we have to stay
 		   compatible with older software
 		 */
-		code=reverse(code,remote->pre_data_bits+
-			     remote->bits+remote->post_data_bits);
+		code=reverse(code,bit_count(remote));
 	}
 	return(code);
 }
 
+int write_message(char *buffer, size_t size, const char *remote_name,
+		  const char *button_name, const char *button_suffix,
+		  ir_code code, int reps)
+{
+	int len;
+	
+#ifdef __GLIBC__
+	/* It seems you can't print 64-bit longs on glibc */
+			
+	len=snprintf(buffer, size,"%08lx%08lx %02x %s%s %s\n",
+		     (unsigned long) (code>>32),
+		     (unsigned long) (code&0xFFFFFFFF),
+		     reps,
+		     button_name, button_suffix,
+		     remote_name);
+#else
+	len=snprintf(buffer, size, "%016llx %02x %s%s %s\n",
+		     code,
+		     reps,
+		     button_name, button_suffix,
+		     remote_name);
+#endif
+	return len;
+}
+
 char *decode_all(struct ir_remote *remotes)
 {
 	struct ir_remote *remote;
 	static char message[PACKET_SIZE+1];
 	ir_code pre,code,post;
 	struct ir_ncode *ncode;
-	int repeat_flag,repeat_state;
+	int repeat_flag;
+	ir_code toggle_bit_mask_state;
 	lirc_t remaining_gap;
+	struct ir_remote *scan;
+	struct ir_ncode *scan_ncode;
 	
 	/* use remotes carefully, it may be changed on SIGHUP */
 	decoding=remote=remotes;
@@ -386,12 +410,12 @@ char *decode_all(struct ir_remote *remotes)
 		
 		if(hw.decode_func(remote,&pre,&code,&post,&repeat_flag,
 				   &remaining_gap) &&
-		   (ncode=get_code(remote,pre,code,post,&repeat_state)))
+		   (ncode=get_code(remote,pre,code,post,&toggle_bit_mask_state)))
 		{
 			int len;
 
-			code=set_code(remote,ncode,repeat_state,repeat_flag,
-				      remaining_gap);
+			code=set_code(remote,ncode,toggle_bit_mask_state,
+				      repeat_flag,remaining_gap);
 			if((has_toggle_mask(remote) &&
 			    remote->toggle_mask_state%2) ||
 			   ncode->current!=NULL)
@@ -400,24 +424,21 @@ char *decode_all(struct ir_remote *remotes)
 				return(NULL);
 			}
 
-#ifdef __GLIBC__
-			/* It seems you can't print 64-bit longs on glibc */
+			for(scan = decoding; scan != NULL; scan = scan->next)
+			{
+				for( scan_ncode = scan->codes; scan_ncode->name != NULL; scan_ncode++)
+				{
+					scan_ncode->current = NULL;
+				}
+			}
+			register_button_press
+				(remote, remote->last_code,
+				 code, remote->reps-(ncode->next ? 1:0));
 			
-			len=snprintf(message,PACKET_SIZE+1,"%08lx%08lx %02x %s %s\n",
-				     (unsigned long)
-				     (code>>32),
-				     (unsigned long)
-				     (code&0xFFFFFFFF),
-				     remote->reps-(ncode->next ? 1:0),
-				     remote->last_code->name,
-				     remote->name);
-#else
-			len=snprintf(message,PACKET_SIZE+1,"%016llx %02x %s %s\n",
-				     code,
-				     remote->reps-(ncode->next ? 1:0),
-				     remote->last_code->name,
-				     remote->name);
-#endif
+			len = write_message(message, PACKET_SIZE+1,
+					    remote->name,
+					    remote->last_code->name, "", code,
+					    remote->reps-(ncode->next ? 1:0));
 			decoding=NULL;
 			if(len>=PACKET_SIZE+1)
 			{
