@@ -1,4 +1,4 @@
-/*      $Id: lircd.h,v 5.11 2002/07/28 08:53:27 lirc Exp $      */
+/*      $Id: lircd.h,v 5.15 2007/05/06 09:46:59 lirc Exp $      */
 
 /****************************************************************************
  ** lircd.h *****************************************************************
@@ -53,14 +53,8 @@ void start_server(mode_t permission,int nodaemon);
   do {} while(0)
 #endif
 
-#ifdef USE_SYSLOG
-#define logprintf syslog
-#define logperror(prio,s) syslog(prio,(s)!=NULL ? "%s: %m\n":"%m\n",s)
-#else
 void logprintf(int prio,char *format_str, ...);
 void logperror(int prio,const char *s);
-#endif
-
 
 void daemonize(void);
 void sigalrm(int sig);
@@ -73,6 +67,7 @@ int send_remote_list(int fd,char *message);
 int send_remote(int fd,char *message,struct ir_remote *remote);
 int send_name(int fd,char *message,struct ir_ncode *code);
 int list(int fd,char *message,char *arguments);
+int set_transmitters(int fd, char *message, char *arguments);
 int send_once(int fd,char *message,char *arguments);
 int send_start(int fd,char *message,char *arguments);
 int send_stop(int fd,char *message,char *arguments);
@@ -80,10 +75,10 @@ int send_core(int fd,char *message,char *arguments,int once);
 int version(int fd,char *message,char *arguments);
 int get_pid(int fd,char *message,char *arguments);
 int get_command(int fd);
+void broadcast_message(const char *message);
 int waitfordata(long maxusec);
 void loop(void);
 
-
 struct protocol_directive
 {
 	char *name;
