@@ -1,4 +1,4 @@
-/*      $Id: receive.c,v 5.29 2005/10/20 18:43:36 lirc Exp $      */
+/*      $Id: receive.c,v 5.34 2007/03/10 21:59:44 lirc Exp $      */
 
 /****************************************************************************
  ** receive.c ***************************************************************
@@ -59,7 +59,7 @@ lirc_t get_next_rec_buffer(lirc_t maxusec)
 		{
 			lirc_t data;
 			
-			data=hw.readdata(2*maxusec<50000 ? 50000:2*maxusec);
+			data=hw.readdata(2*maxusec<100000 ? 100000:2*maxusec);
 			if(!data)
 			{
 				LOGPRINTF(3,"timeout: %u", maxusec);
@@ -298,9 +298,7 @@ inline int expectone(struct ir_remote *remote,int bit)
 {
 	if(is_biphase(remote))
 	{
-		int all_bits = remote->pre_data_bits+
-			remote->bits+
-			remote->post_data_bits;
+		int all_bits = bit_count(remote);
 		ir_code mask;
 		
 		mask=((ir_code) 1)<<(all_bits-1-bit);
@@ -365,9 +363,7 @@ inline int expectzero(struct ir_remote *remote,int bit)
 {
 	if(is_biphase(remote))
 	{
-		int all_bits = remote->pre_data_bits+
-			remote->bits+
-			remote->post_data_bits;
+		int all_bits = bit_count(remote);
 		ir_code mask;
 		
 		mask=((ir_code) 1)<<(all_bits-1-bit);
@@ -492,6 +488,19 @@ inline int get_header(struct ir_remote *remote)
 		unget_rec_buffer(2);
 		return(0);
 	}
+	else if(is_bo(remote))
+	{
+		if(expectpulse(remote, remote->pone) &&
+		   expectspace(remote, remote->sone) &&
+		   expectpulse(remote, remote->pone) &&
+		   expectspace(remote, remote->sone) &&
+		   expectpulse(remote, remote->phead) &&
+		   expectspace(remote, remote->shead))
+		{
+			return 1;
+		}
+		return 0;
+	}
 	if(remote->shead==0)
 	{
 		if(!sync_pending_space(remote)) return 0;
@@ -558,7 +567,7 @@ inline int get_gap(struct ir_remote *remote,lirc_t gap)
 	lirc_t data;
 	
 	LOGPRINTF(2,"sum: %d",rec_buffer.sum);
-	data=get_next_rec_buffer(gap*(100-remote->eps)/100);
+	data=get_next_rec_buffer(gap-gap*remote->eps/100);
 	if(data==0) return(1);
 	if(!is_space(data))
 	{
@@ -898,6 +907,68 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 		set_pending_space(0);
 		return code;
 	}
+	else if(is_bo(remote))
+	{
+		int lastbit = 1;
+		lirc_t deltap,deltas;
+		lirc_t pzero,szero;
+		lirc_t pone,sone;
+		
+		for(i=0; i<bits; i++)
+		{
+			code<<=1;
+			deltap=get_next_pulse(remote->pzero+remote->pone+
+					      remote->ptwo+remote->pthree);
+			deltas=get_next_space(remote->szero+remote->sone+
+					      remote->stwo+remote->sthree);
+			if(deltap==0 || deltas==0) 
+			{
+				logprintf(LOG_ERR,"failed on bit %d",
+					  done+i+1);
+				return((ir_code) -1);
+			}
+			if(lastbit == 1)
+			{
+				pzero = remote->pone;
+				szero = remote->sone;
+				pone = remote->ptwo;
+				sone = remote->stwo;
+			}
+			else
+			{
+				pzero = remote->ptwo;
+				szero = remote->stwo;
+				pone = remote->pthree;
+				sone = remote->sthree;
+			}
+			LOGPRINTF(5, "%lu %lu %lu %lu", pzero, szero, pone, sone);
+			if(expect(remote, deltap, pzero))
+			{
+				if(expect(remote, deltas, szero))
+				{
+					code|=0;
+					lastbit=0;
+					LOGPRINTF(2,"0");
+					continue;
+				}
+			}
+			
+			if(expect(remote, deltap, pone))
+			{
+				if(expect(remote, deltas, sone))
+				{
+					code|=1;
+					lastbit=1;
+					LOGPRINTF(2,"1");
+					continue;
+				}
+			}
+			logprintf(LOG_ERR,"failed on bit %d",
+				  done+i+1);
+			return((ir_code) -1);
+		}
+		return code;
+	}
 	
 	for(i=0;i<bits;i++)
 	{
@@ -1128,12 +1199,10 @@ int receive_decode(struct ir_remote *remote,
 			LOGPRINTF(1,"decoded: %lx",rec_buffer.decoded);
 #                       endif
 			if((hw.rec_mode==LIRC_MODE_CODE &&
-			    hw.code_length<remote->pre_data_bits
-			    +remote->bits+remote->post_data_bits)
+			    hw.code_length<bit_count(remote))
 			   ||
 			   (hw.rec_mode==LIRC_MODE_LIRCCODE && 
-			    hw.code_length!=remote->pre_data_bits
-			    +remote->bits+remote->post_data_bits))
+			    hw.code_length!=bit_count(remote)))
 			{
 				return(0);
 			}
@@ -1157,9 +1226,7 @@ int receive_decode(struct ir_remote *remote,
 			sum=remote->phead+remote->shead+
 				lirc_t_max(remote->pone+remote->sone,
 					   remote->pzero+remote->szero)*
-				(remote->bits+
-				 remote->pre_data_bits+
-				 remote->post_data_bits)+
+				bit_count(remote)+
 				remote->plead+
 				remote->ptrail+
 				remote->pfoot+remote->sfoot+
@@ -1258,7 +1325,7 @@ int receive_decode(struct ir_remote *remote,
 		} /* end of mode specific code */
 	}
 	*prep=pre;*codep=code;*postp=post;
-	if(!has_repeat(remote) &&
+	if((!has_repeat(remote) || remote->reps<remote->min_code_repeat) &&
 	   expect_at_most(remote, sync, remote->remaining_gap))
 		*repeat_flagp=1;
 	else
