@@ -1,4 +1,4 @@
-/*      $Id: transmit.c,v 5.13 2004/02/08 20:42:35 lirc Exp $      */
+/*      $Id: transmit.c,v 5.26 2007/03/10 21:20:07 lirc Exp $      */
 
 /****************************************************************************
  ** transmit.c **************************************************************
@@ -6,7 +6,7 @@
  *
  * functions that prepare IR codes for transmitting
  * 
- * Copyright (C) 1999 Christoph Bartelmus <lirc@bartelmus.de>
+ * Copyright (C) 1999-2004 Christoph Bartelmus <lirc@bartelmus.de>
  *
  */
 
@@ -14,12 +14,18 @@
 # include <config.h>
 #endif
 
+/* if the gap is lower than this value, we will concatenate the
+   signals and send the signal chain at a single blow */
+#define LIRCD_EXACT_GAP_THRESHOLD 10000
+
 #include "lircd.h"
 #include "transmit.h"
 
 extern struct ir_remote *repeat_remote;
 struct sbuf send_buffer;
 
+static void send_signals(lirc_t *signals, int n);
+
 inline void set_bit(ir_code *code,int bit,int data)
 {
 	(*code)&=~((((ir_code) 1)<<bit));
@@ -37,6 +43,7 @@ void init_send_buffer(void)
 
 inline void clear_send_buffer(void)
 {
+	LOGPRINTF(3, "clearing transmit buffer");
 	send_buffer.wptr=0;
 	send_buffer.too_long=0;
 	send_buffer.is_biphase=0;
@@ -49,8 +56,9 @@ inline void add_send_buffer(lirc_t data)
 {
 	if(send_buffer.wptr<WBUF_SIZE)
 	{
+		LOGPRINTF(3, "adding to transmit buffer: %u", data);
 		send_buffer.sum+=data;
-		send_buffer.data[send_buffer.wptr]=data;
+		send_buffer._data[send_buffer.wptr]=data;
 		send_buffer.wptr++;
 	}
 	else
@@ -108,6 +116,20 @@ static inline int bad_send_buffer(void)
 	return(0);
 }
 
+static inline void flush_send_buffer(void)
+{
+	if(send_buffer.pendingp>0)
+	{
+		add_send_buffer(send_buffer.pendingp);
+		send_buffer.pendingp=0;
+	}
+	if(send_buffer.pendings>0)
+	{
+		add_send_buffer(send_buffer.pendings);
+		send_buffer.pendings=0;
+	}
+}
+
 static inline void sync_send_buffer(void)
 {
 	if(send_buffer.pendingp>0)
@@ -155,25 +177,63 @@ inline void send_trail(struct ir_remote *remote)
 inline void send_data(struct ir_remote *remote,ir_code data,int bits,int done)
 {
 	int i;
-	int all_bits = remote->pre_data_bits+
-		remote->bits+
-		remote->post_data_bits;
+	int all_bits = bit_count(remote);
 	ir_code mask;
-	
-	if(remote->toggle_bit>0)
+	if(is_rcmm(remote))
 	{
-		if(remote->toggle_bit>done &&
-		   remote->toggle_bit<=done+bits)
+		data=reverse(data,bits);
+		mask=1<<(all_bits-1-done);
+		if(bits%2 || done%2)
 		{
-			set_bit(&data,done+bits-remote->toggle_bit,
-				remote->repeat_state);
+			logprintf(LOG_ERR,"invalid bit number.");
+			return;
 		}
+		for(i=0;i<bits;i+=2,mask>>=2)
+		{
+			switch(data&3)
+			{
+			case 0:
+				send_pulse(remote->pzero);
+				send_space(remote->szero);
+				break;
+			/* 2 and 1 swapped due to reverse() */
+			case 2:
+				send_pulse(remote->pone);
+				send_space(remote->sone);
+				break;
+			case 1:
+				send_pulse(remote->ptwo);
+				send_space(remote->stwo);
+				break;
+			case 3:
+				send_pulse(remote->pthree);
+				send_space(remote->sthree);
+				break;
+			}
+			data=data>>2;
+		}
+		return;
 	}
 
 	data=reverse(data,bits);
-	mask=1<<(all_bits-1-done);
+	mask=((ir_code) 1)<<(all_bits-1-done);
 	for(i=0;i<bits;i++,mask>>=1)
 	{
+		if(has_toggle_bit_mask(remote) && mask&remote->toggle_bit_mask)
+		{
+			data &= ~((ir_code) 1);
+			if(remote->toggle_bit_mask_state&mask)
+			{
+				data |= (ir_code) 1;
+			}
+			
+		}
+		if(has_toggle_mask(remote) &&
+		   mask&remote->toggle_mask &&
+		   remote->toggle_mask_state%2)
+		{
+			data ^= 1;
+		}
 		if(data&1)
 		{
 			if(is_biphase(remote))
@@ -190,6 +250,11 @@ inline void send_data(struct ir_remote *remote,ir_code data,int bits,int done)
 					send_pulse(remote->pone);
 				}
 			}
+			else if(is_space_first(remote))
+			{
+				send_space(remote->sone);
+				send_pulse(remote->pone);
+			}
 			else
 			{
 				send_pulse(remote->pone);
@@ -203,6 +268,11 @@ inline void send_data(struct ir_remote *remote,ir_code data,int bits,int done)
 				send_pulse(2*remote->pzero);
 				send_space(2*remote->szero);
 			}
+			else if(is_space_first(remote))
+			{
+				send_space(remote->szero);
+				send_pulse(remote->pzero);
+			}
 			else
 			{
 				send_pulse(remote->pzero);
@@ -248,19 +318,19 @@ inline void send_repeat(struct ir_remote *remote)
 	send_trail(remote);
 }
 
-inline void send_code(struct ir_remote *remote,ir_code code)
+inline void send_code(struct ir_remote *remote,ir_code code, int repeat)
 {
-	if(repeat_remote==NULL || !(remote->flags&NO_HEAD_REP))
+	if(!repeat || !(remote->flags&NO_HEAD_REP))
 		send_header(remote);
 	send_lead(remote);
 	send_pre(remote);
 	send_data(remote,code,remote->bits,remote->pre_data_bits);
 	send_post(remote);
 	send_trail(remote);
-	if(repeat_remote==NULL || !(remote->flags&NO_FOOT_REP))
+	if(!repeat || !(remote->flags&NO_FOOT_REP))
 		send_foot(remote);
 	
-	if(repeat_remote==NULL &&
+	if(!repeat &&
 	   remote->flags&NO_HEAD_REP &&
 	   remote->flags&CONST_LENGTH)
 	{
@@ -268,10 +338,22 @@ inline void send_code(struct ir_remote *remote,ir_code code)
 	}
 }
 
+static void send_signals(lirc_t *signals, int n)
+{
+	int i;
+	
+	for(i=0; i<n; i++)
+	{
+		add_send_buffer(signals[i]);
+	}
+}
+
 int init_send(struct ir_remote *remote,struct ir_ncode *code)
 {
-	if(is_rcmm(remote) || is_grundig(remote) || 
-	   is_goldstar(remote) || is_serial(remote))
+	int i, repeat=0;
+	
+	if(is_grundig(remote) || 
+	   is_goldstar(remote) || is_serial(remote) || is_bo(remote))
 	{
 		logprintf(LOG_ERR,"sorry, can't send this protocol yet");
 		return(0);
@@ -281,7 +363,17 @@ int init_send(struct ir_remote *remote,struct ir_ncode *code)
 	{
 		send_buffer.is_biphase=1;
 	}
-	if(repeat_remote!=NULL && has_repeat(remote))
+	if(repeat_remote==NULL)
+	{
+		remote->repeat_countdown=remote->min_repeat;
+	}
+	else
+	{
+		repeat = 1;
+	}
+	
+ init_send_loop:
+	if(repeat && has_repeat(remote))
 	{
 		if(remote->flags&REPEAT_HEADER && has_header(remote))
 		{
@@ -293,26 +385,46 @@ int init_send(struct ir_remote *remote,struct ir_ncode *code)
 	{
 		if(!is_raw(remote))
 		{
+			ir_code next_code;
+			
+			if(code->transmit_state == NULL)
+			{
+				next_code = code->code;
+			}
+			else
+			{
+				next_code = code->transmit_state->code;
+			}
+			send_code(remote, next_code, repeat);
 			if(has_toggle_mask(remote))
 			{
-				if(remote->toggle_mask_state%2)
-				{
-					send_code(remote,code->code^
-						  remote->toggle_mask);
-				}
-				else
-				{
-					send_code(remote,code->code);
-				}
 				remote->toggle_mask_state++;
 				if(remote->toggle_mask_state==4)
 				{
 					remote->toggle_mask_state=2;
 				}
 			}
+			send_buffer.data=send_buffer._data;
+		}
+		else
+		{
+			if(code->signals==NULL)
+			{
+				logprintf(LOG_ERR, "no signals for raw send");
+				return 0;
+			}
+			if(send_buffer.wptr>0)
+			{
+				send_signals(code->signals, code->length);
+			}
 			else
 			{
-				send_code(remote,code->code);
+				send_buffer.data=code->signals;
+				send_buffer.wptr=code->length;
+				for(i=0; i<code->length; i++)
+				{
+					send_buffer.sum+=code->signals[i];
+				}
 			}
 		}
 	}
@@ -322,7 +434,11 @@ int init_send(struct ir_remote *remote,struct ir_ncode *code)
 		logprintf(LOG_ERR,"buffer too small");
 		return(0);
 	}
-	if(is_const(remote))
+	if(has_repeat_gap(remote) && repeat && has_repeat(remote))
+	{
+		remote->remaining_gap=remote->repeat_gap;
+	}
+	else if(is_const(remote))
 	{
 		if(remote->gap>send_buffer.sum)
 		{
@@ -338,16 +454,48 @@ int init_send(struct ir_remote *remote,struct ir_ncode *code)
 	}
 	else
 	{
-		if(has_repeat_gap(remote) &&
-		   repeat_remote!=NULL &&
-		   has_repeat(remote))
+		remote->remaining_gap=remote->gap;
+	}
+	/* update transmit state */
+	if(code->next != NULL)
+	{
+		if(code->transmit_state == NULL)
 		{
-			remote->remaining_gap=remote->repeat_gap;
+			code->transmit_state = code->next;
 		}
 		else
 		{
-			remote->remaining_gap=remote->gap;
+			code->transmit_state = code->transmit_state->next;
 		}
 	}
+	if((remote->repeat_countdown>0 || code->transmit_state != NULL) &&
+	   remote->remaining_gap<LIRCD_EXACT_GAP_THRESHOLD)
+	{
+		if(send_buffer.data!=send_buffer._data)
+		{
+			lirc_t *signals;
+			int n;
+			
+			LOGPRINTF(1, "unrolling raw signal optimisation");
+			signals=send_buffer.data;
+			n=send_buffer.wptr;
+			send_buffer.data=send_buffer._data;
+			send_buffer.wptr=0;
+			
+			send_signals(signals, n);
+		}
+		LOGPRINTF(1, "concatenating low gap signals");
+		if(code->next == NULL || code->transmit_state == NULL)
+		{
+			remote->repeat_countdown--;
+		}
+		send_space(remote->remaining_gap);
+		flush_send_buffer();
+		send_buffer.sum=0;
+		
+		repeat = 1;
+		goto init_send_loop;
+	}
+	LOGPRINTF(3, "transmit buffer ready");
 	return(1);
 }
