@@ -46,7 +46,7 @@
 # include <config.h>
 #endif
  
-#include <linux/config.h>
+#include <linux/autoconf.h>
 
 #if !defined(LIRC_ON_SA1100) && !defined(CONFIG_SERIAL_MODULE)
 #warning "******************************************"
@@ -75,7 +75,7 @@
 #include <linux/delay.h>
 #include <linux/poll.h>
 #include <asm/system.h>
-#include <asm/segment.h>
+#include <asm/uaccess.h>
 #include <asm/io.h>
 #include <asm/irq.h>
 #include <asm/fcntl.h>
@@ -220,8 +220,12 @@ static int init_chrdev(void);
 static void drop_chrdev(void);
 #endif
 	/* Hardware */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 static irqreturn_t sir_interrupt(int irq, void * dev_id,
 				 struct pt_regs * regs);
+#else
+static irqreturn_t sir_interrupt(int irq, void * dev_id);
+#endif
 static void send_space(unsigned long len);
 static void send_pulse(unsigned long len);
 static int init_hardware(void);
@@ -537,6 +541,7 @@ static struct lirc_plugin plugin = {
        set_use_inc:    set_use_inc,
        set_use_dec:    set_use_dec,
        fops:           &lirc_fops,
+       dev:            NULL,
        owner:          THIS_MODULE,
 };
 
@@ -601,8 +606,12 @@ static void sir_timeout(unsigned long data)
 	spin_unlock_irqrestore(&timer_lock, flags);		
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 static irqreturn_t sir_interrupt(int irq, void * dev_id,
 				 struct pt_regs * regs)
+#else
+static irqreturn_t sir_interrupt(int irq, void * dev_id)
+#endif
 {
 	unsigned char data;
 	struct timeval curr_tv;
