@@ -1,6 +1,6 @@
 #! /bin/sh
 
-LIRC_VERSION="0.7.0"
+LIRC_VERSION="0.8.2"
 
 #############################################################################
 ## Default Values
@@ -25,7 +25,7 @@ SELECTED_DRIVER=""
 DRIVER_PARAMETER="com1"
 DRIVER_PARAM_TYPE=""
 SOFT_CARRIER="on"
-TRANSMITTER="off"
+TRANSMITTER="on"
 IGOR="off"
 TIMER=65536
 X11_WINDOWS="on"
@@ -61,7 +61,7 @@ GetSelectedDriver ()
     LPT1="off"; LPT2="off"; LPT3="off"; USER="off"
     IRTTY="none"
 
-    if   test "$DRIVER_PARAMETER" = "btty"; then IRTTY="/dev/btty"; LIRC_PORT="none"; LIRC_IRQ="none"
+    if   test "$DRIVER_PARAMETER" = "btty"; then IRTTY="/dev/rfcomm0"; LIRC_PORT="none"; LIRC_IRQ="none"
     elif test "$DRIVER_PARAMETER" = "ttyUSB1"; then COM1="on"; IRTTY="/dev/ttyUSB0"; LIRC_PORT="none"; LIRC_IRQ="none"
     elif test "$DRIVER_PARAMETER" = "ttyUSB2"; then COM2="on"; IRTTY="/dev/ttyUSB1"; LIRC_PORT="none"; LIRC_IRQ="none"
     elif test "$DRIVER_PARAMETER" = "ttyUSB3"; then COM3="on"; IRTTY="/dev/ttyUSB2"; LIRC_PORT="none"; LIRC_IRQ="none"
@@ -265,7 +265,7 @@ DriverOptions ()
 
 ConfigDriver ()
     {
-    . setup-driver.sh
+    . ./setup-driver.sh
     if test "$?" = "0"; then
         GetSelectedDriver
         SetPortAndIrq
@@ -318,6 +318,7 @@ SaveConfig ()
     echo "LIRC_IRQ=$LIRC_IRQ" >>$CONFIG
     echo "LIRC_MAJOR=$LIRC_MAJOR" >>$CONFIG
     echo "IRTTY=$IRTTY" >>$CONFIG
+    echo "DRIVER_PARAM_TYPE=$DRIVER_PARAM_TYPE" >>$CONFIG
     echo "DRIVER_PARAMETER=$DRIVER_PARAMETER" >>$CONFIG
     echo "SOFT_CARRIER=$SOFT_CARRIER" >>$CONFIG
     echo "TRANSMITTER=$TRANSMITTER" >>$CONFIG
@@ -331,7 +332,7 @@ SaveConfig ()
     echo "DYNCODES=$DYNCODES" >>$CONFIG
     chmod 666 $CONFIG
 
-    echo '#!/bin/bash' >$START
+    echo '#! /bin/sh' >$START
     echo >>$START
     echo "./configure \\" >>$START
     echo "--with-moduledir=/lib/modules/`uname -r`/misc \\" >>$START
@@ -379,7 +380,10 @@ fi
 if test -f $CONFIG; then
     {
     echo "Loading saved configuration from $CONFIG"
-    . $CONFIG
+    case $CONFIG in
+    */*) . $CONFIG ;;
+    *) . ./$CONFIG ;;
+    esac
     sleep 1
     }
 fi
